{===============================================================================
  RzDBEmp Unit

  This unit implements the TRzDBEmployee data-aware business component.  This
  component is responsible to defining the business rules associated with
  employees data stored in the Employee table of the DBDemos database.

  Copyright  1995, 1996 by Raize Software Solutions, Inc.
===============================================================================}


unit RzDBEmp;

interface

uses
  Classes, DB, DBTables, SysUtils, RzBizCmp, RzErrDlg;

type
  EExceedSalaryCap = class( Exception );

  TRzDBEmployee = class( TRzBusinessComponent )
  private
    { Field Objects }
    FEmpNo : TIntegerField;
    FLastName : TStringField;
    FFirstName : TStringField;
    FHireDate : TDateTimeField;
    FSalary : TFloatField;
    { Calculated Field Object }
    FFullName : TStringField;
    FMonthlySal : TFloatField;
    { Business Property }
    FSalaryCap : Single;
  protected
    procedure CreateFields; override;
    procedure CalculateFields; override;
    procedure Validate; override;
    procedure FinishInsert; override;
  public
    { Methods Implementing Business Rules }
    function GetFullName : string;
    function YearsOfService : Single;
    function TotalVacationDays : Integer;

    { Specify properties to enforce read-only attribute }
    property EmpNo : TIntegerField
      read FEmpNo;

    property LastName : TStringField
      read FLastName;

    property FirstName : TStringField
      read FFirstName;

    property FullName : TStringField
      read FFullName;

    property HireDate : TDateTimeField
      read FHireDate;

    property Salary : TFloatField
      read FSalary;

    property MonthlySal: TFloatField
      read FMonthlySal;

  published
    property SalaryCap : Single
      read FSalaryCap
      write FSalaryCap;
  end;


procedure Register;


implementation

uses
  Dialogs;


{===========================}
{== TRzDBEmployee Methods ==}
{===========================}


{===============================================================================
  TRzDBEmployee.CreateFields

  This method specifies the Field Objects that are to be used by the
  Employee Business Component.
===============================================================================}

procedure TRzDBEmployee.CreateFields;
begin
  FEmpNo := CreateMainField( 'EmpNo' ) as TIntegerField;
  FEmpNo.Visible := False;

  FLastName := CreateMainField( 'LastName' ) as TStringField;
  FLastName.DisplayLabel := 'Last Name';
  FLastName.Visible := False;

  FFirstName := CreateMainField( 'FirstName' ) as TStringField;
  FFirstName.DisplayLabel := 'First Name';
  FFirstName.Visible := False;

  FFullName := CreateMainCalcField( 'FullName', TStringField, 30 ) as TStringField;
  FFullName.DisplayLabel := 'Full Name';

  FHireDate := CreateMainField( 'HireDate' ) as TDateTimeField;
  FHireDate.DisplayLabel := 'Hire Date';
  FHireDate.EditMask := '!99/99/00;1;_';

  FSalary := CreateMainField( 'Salary' ) as TFloatField;
  FSalary.Currency := True;

  FMonthlySal := CreateMainCalcField( 'MonthlySal', TFloatField, 0 ) as TFloatField;
  FMonthlySal.DisplayLabel := 'Monthly Salary';
  FMonthlySal.Currency := True;
end; {= TRzDBEmployeeObject.CreateFields =}


function TRzDBEmployee.GetFullName : string;
begin
  Result := FFirstName.Value + ' ' + FLastName.Value;
end;


function TRzDBEmployee.YearsOfService : Single;
begin
  Result := Trunc( Now - FHireDate.Value ) / 365;
end;


{===============================================================================
  TRzDBEmployee.TotalVacationDays

  This method calculates the number of vacation days available for the current
  year.  The number is based on how long the employee has been with the company.
  The number is scaled according to the employee's years of service.  The scale
  is as follows:

  Less than 6 months = 0 days
  Less than 5 years  = 10 days (2 Weeks)
  5 to 9 years       = 15 days (3 Weeks)
  10 to 14 years     = 20 days (4 Weeks)
  15 to 19 years     = 25 days (5 Weeks)
  ...
===============================================================================}

function TRzDBEmployee.TotalVacationDays : Integer;
var
  YOS : Real;
begin
  YOS := YearsOfService;
  if YOS < 0.5 then
    Result := 0
  else
    Result := ( Trunc( YOS ) div 5 + 2 ) * 5;
end;


{===============================================================================
  TRzDBEmployee.Validate

  This method gets called whenever the Salary column in the table is updated.
  If the Salary exceeds the salary cap, an exception is raised to abort the
  current process.  For example, if the salary cap is set to 70,000 and the user
  enters 85,000 and then tries to Post the change, the
  exception will cause the Post to fail.
===============================================================================}

procedure TRzDBEmployee.Validate;
begin
  if ( FSalaryCap <> 0 ) and ( FSalary.Value > FSalaryCap ) then
    ErrorHandler.Add( Format( 'Salary cannot exceed %m.', [ FSalaryCap ] ),
                      etValidation );

  if FirstName.Value = '' then
    ErrorHandler.Add( 'First Name is not specified.', etWarning );
end;


{===============================================================================
  TRzDBEmployee.CalculateFields

  Override the CalculateFields method to have the business component determine
  calculated field values.
===============================================================================}

procedure TRzDBEmployee.CalculateFields;
begin
  FMonthlySal.Value := FSalary.Value / 12;
  FFullName.Value := GetFullName;
end;


{===============================================================================
  TRzDBEmployee.FinishInsert

  Override the FinishInsert method to specify initial values to be used when
  a new record is created.
===============================================================================}

procedure TRzDBEmployee.FinishInsert;
begin
  FHireDate.Value := Date;
end;


{========================}
{== Register Procedure ==}
{========================}

procedure Register;
begin
  RegisterComponents( 'RzData', [ TRzDBEmployee ] );
end;


end.
