{===============================================================================
  RzErrHnd Unit

  This unit implements the general purpose TRzErrorHandler nonvisual component.
  The Raize Business Component Architecture utilizes the TRzErrorHandler
  component during validation of a data-aware business component.  By simply
  attaching an error handler to a business component, the error handler will be
  used to display validation errors and validation warnings to the user.

  Displaying the error dialog box is handled internally by the business
  component.  However, the dialog box will not be displayed if there are no
  validation errors or warnings.  To add validation errors or warnings to the
  error handler, the Add method is used.  This method takes a message
  string and an error type.  For validation, the two important types are
  etWarning and etValidation.

  When validation is completed, the business component checks the associated
  error handler's error list.  If it is empty, the record is posted.  If not,
  the error handler dialog box is displayed.  The dialog box displays *all*
  validation errors and warnings in a list box.  If the list contains only
  warnings, a Proceed and Cancel button are displayed.  If the list contains
  an error, regardless of whether it contains any warnings, an OK button is
  displayed.

  Copyright  1995-1996 by Raize Software Solutions, Inc.
===============================================================================}

unit RzErrHnd;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, RzErrDlg;

const
  mrProceed = 20;                                    { Modal Result for Dialog }

type
  TValidationErrors = ( veErrorsOnly, veWarningsOnly, veErrorsAndWarnings );


  TRzErrorHandler = class( TComponent )
  private
    FButtonGlyphs : Boolean;
    FCaption : string;
    FFont : TFont;
    FUserResponse : TModalResult;
    procedure SetFont( Value : TFont );
  public
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
    procedure Clear; virtual;
    function  GetTypeOfErrors : TValidationErrors;
    procedure Add( Msg : string; ErrorType : TErrorType );
    function DisplayErrorList : TModalResult;
    function  HandleErrors : Boolean; virtual;

    property  UserResponse : TModalResult
      read FUserResponse
      write FUserResponse;
  published
    property ButtonGlyphs : Boolean
      read FButtonGlyphs
      write FButtonGlyphs
      default False;

    property Caption : string
      read FCaption
      write FCaption;

    property Font : TFont
      read FFont
      write SetFont;
  end;


procedure Register;

implementation


{=============================}
{== TRzErrorHandler Methods ==}
{=============================}

constructor TRzErrorHandler.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  FFont := TFont.Create;
  if Owner is TForm then                               { If Owner is a form... }
    FFont.Assign( TForm( Owner ).Font )       { Use the form's font by default }
  else
  begin
    FFont := TFont.Create;
    FFont.Name := 'MS Sans Serif';
    FFont.Size := 8;
    FFont.Style := [];
  end;
  FButtonGlyphs := False;

  FUserResponse := mrProceed;
  FrmErrorDlg := TFrmErrorDlg.Create( Self );
end;


destructor TRzErrorHandler.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;


procedure TRzErrorHandler.SetFont( Value : TFont );
begin
  FFont.Assign( Value );
end;


procedure TRzErrorHandler.Clear;
begin
  FrmErrorDlg.ClearMessages;
end;


function TRzErrorHandler.GetTypeOfErrors : TValidationErrors;
var
  I, ErrorCount, WarningCount : Integer;
begin
  ErrorCount := 0;
  WarningCount := 0;

  for I := 0 to Pred( FrmErrorDlg.LstErrors.Items.Count ) do
  begin
    case TErrorType( FrmErrorDlg.LstErrors.Items.Objects[ I ] ) of
      etWarning:
        Inc( WarningCount );

      etValidation:
        Inc( ErrorCount );
    end;
  end;

  if ( ErrorCount > 0 ) and ( WarningCount > 0 ) then
    Result := veErrorsAndWarnings
  else if ErrorCount > 0 then
    Result := veErrorsOnly
  else
    Result := veWarningsOnly;
end;


function TRzErrorHandler.DisplayErrorList : TModalResult;
begin
  if FrmErrorDlg.NumberOfErrors = 0 then
  begin
    MessageDlg( 'No errors to display', mtInformation, [ mbOK ], 0 );
    Result := mrCancel;
    Exit;
  end;

  if FCaption <> '' then
    FrmErrorDlg.Caption := FCaption;
  FrmErrorDlg.SetButtons( GetTypeOfErrors = veWarningsOnly );

  with FrmErrorDlg do
  begin
    Font := FFont;

    if FButtonGlyphs then
    begin
      BtnOK.Glyph.Handle := LoadBitmap( HInstance, 'RZEH_OK' );
      BtnCancel.Glyph.Handle := LoadBitmap( HInstance, 'RZEH_CANCEL' );
      BtnProceed.Glyph.Handle := LoadBitmap( HInstance, 'RZEH_OK' );
    end
    else
    begin
      BtnOK.Glyph := nil;
      BtnCancel.Glyph := nil;
      BtnProceed.Glyph := nil;
    end;

    if not NewStyleControls or FButtonGlyphs then
    begin
      BtnOK.Height := 26;
      BtnCancel.Height := 26;
      BtnProceed.Height := 26;
    end;
  end; { with }

  Result := FrmErrorDlg.ShowModal;
  FUserResponse := Result;
end;


function TRzErrorHandler.HandleErrors : Boolean;
begin
  if FrmErrorDlg.NumberOfErrors > 0 then
    DisplayErrorList;
  Result := FrmErrorDlg.NumberOfErrors <> 0;
end;


procedure TRzErrorHandler.Add( Msg : string; ErrorType : TErrorType );
begin
  FrmErrorDlg.AddError( Msg, ErrorType );
end;


{========================}
{== Register Procedure ==}
{========================}

procedure Register;
begin
  RegisterComponents( 'RzData', [ TRzErrorHandler ] );
end;

end.
